@@ -453,12 +453,22 @@ class IteratorsTestCase(unittest.TestCase):
         self.assertEqual(["c", "b", "a"], list(it))
 
     def test_flatten(self):
+        # flatten nested lists
         self.assertEqual([1,2,3,4], list(iters.flatten([[1,2], [3,4]])))
         self.assertEqual([1,2,3,4,5,6], list(iters.flatten([[1,2], [3, [4,5,6]]])))
-        # flat list should return themself 
+        # flatten nested tuples, sets, and frozen sets
+        self.assertEqual([1,2,3,4,5,6], list(iters.flatten(((1,2), (3, (4,5,6))))))
+        self.assertEqual([1,2,3], list(iters.flatten(set([1, frozenset([2,3])]))))
+        # flatten nested generators
+        generators = ((num + 1 for num in range(0, n)) for n in range(1, 4))
+        self.assertEqual([1,1,2,1,2,3], list(iters.flatten(generators)))
+        # flat list should return itself
         self.assertEqual([1,2,3], list(iters.flatten([1,2,3])))
-        # string is not a list
+        # Don't flatten strings, bytes, or bytearrays
         self.assertEqual([2,"abc",1], list(iters.flatten([2,"abc",1])))
+        self.assertEqual([2, b'abc', 1], list(iters.flatten([2, b'abc', 1])))
+        self.assertEqual([2, bytearray(b'abc'), 1], 
+                         list(iters.flatten([2, bytearray(b'abc'), 1])))
 
     def test_accumulate(self):
         self.assertEqual([1,3,6,10,15], list(iters.accumulate([1,2,3,4,5])))
@@ -637,6 +647,7 @@ class OptionTestCase(unittest.TestCase, InstanceChecker):
 class TrampolineTestCase(unittest.TestCase):
 
     def test_tco_decorator(self):
+
         def recur_accumulate(origin, f=operator.add, acc=0):
             n = next(origin, None)
             if n is None: return acc
@@ -661,5 +672,19 @@ class TrampolineTestCase(unittest.TestCase):
 
         self.assertEqual(sum(range(limit)), tco_accumulate(iter(range(limit))))
 
+    def test_tco_different_functions(self):
+
+        @recur.tco
+        def recur_inc2(curr, acc=0):
+            if curr == 0: return False, acc
+            return recur_dec, (curr-1, acc+2)
+
+        @recur.tco
+        def recur_dec(curr, acc=0):
+            if curr == 0: return False, acc
+            return recur_inc2, (curr-1, acc-1)
+
+        self.assertEqual(5000, recur_inc2(10000))
+
 if __name__ == '__main__':
     unittest.main()
